/*
 * Copyright 2014 Red Hat, Inc.
 *
 * Red Hat licenses this file to you under the Apache License, version 2.0
 * (the "License"); you may not use this file except in compliance with the
 * License.  You may obtain a copy of the License at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

package io.vertx.rxjava.cassandra;

import java.util.Map;
import rx.Observable;
import rx.Single;
import com.datastax.driver.core.Row;
import io.vertx.cassandra.CassandraClientOptions;
import com.datastax.driver.core.PreparedStatement;
import java.util.List;
import io.vertx.core.AsyncResult;
import io.vertx.core.Handler;
import com.datastax.driver.core.Statement;

/**
 * Eclipse Vert.x Cassandra client.
 *
 * <p/>
 * NOTE: This class has been automatically generated from the {@link io.vertx.cassandra.CassandraClient original} non RX-ified interface using Vert.x codegen.
 */

@io.vertx.lang.rx.RxGen(io.vertx.cassandra.CassandraClient.class)
public class CassandraClient {

  @Override
  public String toString() {
    return delegate.toString();
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) return true;
    if (o == null || getClass() != o.getClass()) return false;
    CassandraClient that = (CassandraClient) o;
    return delegate.equals(that.delegate);
  }
  
  @Override
  public int hashCode() {
    return delegate.hashCode();
  }

  public static final io.vertx.lang.rx.TypeArg<CassandraClient> __TYPE_ARG = new io.vertx.lang.rx.TypeArg<>(    obj -> new CassandraClient((io.vertx.cassandra.CassandraClient) obj),
    CassandraClient::getDelegate
  );

  private final io.vertx.cassandra.CassandraClient delegate;
  
  public CassandraClient(io.vertx.cassandra.CassandraClient delegate) {
    this.delegate = delegate;
  }

  public io.vertx.cassandra.CassandraClient getDelegate() {
    return delegate;
  }

  /**
   * Like {@link io.vertx.rxjava.cassandra.CassandraClient#createNonShared}  but with default client options.
   * @param vertx 
   * @return 
   */
  public static io.vertx.rxjava.cassandra.CassandraClient createNonShared(io.vertx.rxjava.core.Vertx vertx) { 
    io.vertx.rxjava.cassandra.CassandraClient ret = io.vertx.rxjava.cassandra.CassandraClient.newInstance(io.vertx.cassandra.CassandraClient.createNonShared(vertx.getDelegate()));
    return ret;
  }

  /**
   * Create a Cassandra client which maintains its own data source.
   * <p>
   * It is not recommended to create several non shared clients in an application.
   * Your application should either use only single non shared client, or use shared client.
   * This is because {@link io.vertx.rxjava.cassandra.CassandraClient} backed by {@link com.datastax.driver.core.Session}.
   * And Datastax does not recommended to have several {@link com.datastax.driver.core.Session}
   * instances. Better to have only one, and share it.
   * @param vertx the Vert.x instance
   * @param cassandraClientOptions the options
   * @return the client
   */
  public static io.vertx.rxjava.cassandra.CassandraClient createNonShared(io.vertx.rxjava.core.Vertx vertx, CassandraClientOptions cassandraClientOptions) { 
    io.vertx.rxjava.cassandra.CassandraClient ret = io.vertx.rxjava.cassandra.CassandraClient.newInstance(io.vertx.cassandra.CassandraClient.createNonShared(vertx.getDelegate(), cassandraClientOptions));
    return ret;
  }

  /**
   * Like {@link io.vertx.rxjava.cassandra.CassandraClient#createShared}, but with default client options and datasource.
   * @param vertx 
   * @return 
   */
  public static io.vertx.rxjava.cassandra.CassandraClient createShared(io.vertx.rxjava.core.Vertx vertx) { 
    io.vertx.rxjava.cassandra.CassandraClient ret = io.vertx.rxjava.cassandra.CassandraClient.newInstance(io.vertx.cassandra.CassandraClient.createShared(vertx.getDelegate()));
    return ret;
  }

  /**
   * Like {@link io.vertx.rxjava.cassandra.CassandraClient#createShared}, but with default client options.
   * @param vertx 
   * @param datasourceName 
   * @return 
   */
  public static io.vertx.rxjava.cassandra.CassandraClient createShared(io.vertx.rxjava.core.Vertx vertx, String datasourceName) { 
    io.vertx.rxjava.cassandra.CassandraClient ret = io.vertx.rxjava.cassandra.CassandraClient.newInstance(io.vertx.cassandra.CassandraClient.createShared(vertx.getDelegate(), datasourceName));
    return ret;
  }

  /**
   * Like {@link io.vertx.rxjava.cassandra.CassandraClient#createShared}, but with datasource name.
   * @param vertx 
   * @param cassandraClientOptions 
   * @return 
   */
  public static io.vertx.rxjava.cassandra.CassandraClient createShared(io.vertx.rxjava.core.Vertx vertx, CassandraClientOptions cassandraClientOptions) { 
    io.vertx.rxjava.cassandra.CassandraClient ret = io.vertx.rxjava.cassandra.CassandraClient.newInstance(io.vertx.cassandra.CassandraClient.createShared(vertx.getDelegate(), cassandraClientOptions));
    return ret;
  }

  /**
   * Create a Cassandra client which shares its data source with any other Cassandra clients created with the same
   * data source name.
   * @param vertx the Vert.x instance
   * @param datasourceName the data source name
   * @param cassandraClientOptions the options
   * @return the client
   */
  public static io.vertx.rxjava.cassandra.CassandraClient createShared(io.vertx.rxjava.core.Vertx vertx, String datasourceName, CassandraClientOptions cassandraClientOptions) { 
    io.vertx.rxjava.cassandra.CassandraClient ret = io.vertx.rxjava.cassandra.CassandraClient.newInstance(io.vertx.cassandra.CassandraClient.createShared(vertx.getDelegate(), datasourceName, cassandraClientOptions));
    return ret;
  }

  /**
   * Connect to a Cassandra service.
   * @return current Cassandra client instance
   */
  public io.vertx.rxjava.cassandra.CassandraClient connect() { 
    delegate.connect();
    return this;
  }

  /**
   * @return whether this Cassandra client instance connected.
   */
  public boolean isConnected() { 
    boolean ret = delegate.isConnected();
    return ret;
  }

  /**
   * Connect to a Cassandra service.
   * @param connectHandler handler called when asynchronous connect call ends
   * @return current Cassandra client instance
   */
  public io.vertx.rxjava.cassandra.CassandraClient connect(Handler<AsyncResult<Void>> connectHandler) { 
    delegate.connect(connectHandler);
    return this;
  }

    /**
   * Connect to a Cassandra service.
   * @return current Cassandra client instance
   */
  public Single<Void> rxConnect() { 
    return Single.create(new io.vertx.rx.java.SingleOnSubscribeAdapter<>(fut -> {
      connect(fut);
    }));
  }

  /**
   * Connect to a Cassandra service.
   * @param keyspace The name of the keyspace to use for the created connection.
   * @param connectHandler handler called when asynchronous connect call ends
   * @return current Cassandra client instance
   */
  public io.vertx.rxjava.cassandra.CassandraClient connect(String keyspace, Handler<AsyncResult<Void>> connectHandler) { 
    delegate.connect(keyspace, connectHandler);
    return this;
  }

    /**
   * Connect to a Cassandra service.
   * @param keyspace The name of the keyspace to use for the created connection.
   * @return current Cassandra client instance
   */
  public Single<Void> rxConnect(String keyspace) { 
    return Single.create(new io.vertx.rx.java.SingleOnSubscribeAdapter<>(fut -> {
      connect(keyspace, fut);
    }));
  }

  /**
   * Execute the query and provide a handler for consuming results.
   * @param query the query to execute
   * @param resultHandler handler called when result of execution is present, but can be not fully fetched
   * @return current Cassandra client instance
   */
  public io.vertx.rxjava.cassandra.CassandraClient execute(String query, Handler<AsyncResult<io.vertx.rxjava.cassandra.ResultSet>> resultHandler) { 
    delegate.execute(query, new Handler<AsyncResult<io.vertx.cassandra.ResultSet>>() {
      public void handle(AsyncResult<io.vertx.cassandra.ResultSet> ar) {
        if (ar.succeeded()) {
          resultHandler.handle(io.vertx.core.Future.succeededFuture(io.vertx.rxjava.cassandra.ResultSet.newInstance(ar.result())));
        } else {
          resultHandler.handle(io.vertx.core.Future.failedFuture(ar.cause()));
        }
      }
    });
    return this;
  }

    /**
   * Execute the query and provide a handler for consuming results.
   * @param query the query to execute
   * @return current Cassandra client instance
   */
  public Single<io.vertx.rxjava.cassandra.ResultSet> rxExecute(String query) { 
    return Single.create(new io.vertx.rx.java.SingleOnSubscribeAdapter<>(fut -> {
      execute(query, fut);
    }));
  }

  /**
   * Executes the given SQL <code>SELECT</code> statement which returns the results of the query as a read stream.
   * @param sql the SQL to execute. For example <code>SELECT * FROM table ...</code>.
   * @param rowStreamHandler the handler which is called once the operation completes. It will return an instance of {@link io.vertx.rxjava.cassandra.CassandraRowStream}.
   * @return current Cassandra client instance
   */
  public io.vertx.rxjava.cassandra.CassandraClient queryStream(String sql, Handler<AsyncResult<io.vertx.rxjava.cassandra.CassandraRowStream>> rowStreamHandler) { 
    delegate.queryStream(sql, new Handler<AsyncResult<io.vertx.cassandra.CassandraRowStream>>() {
      public void handle(AsyncResult<io.vertx.cassandra.CassandraRowStream> ar) {
        if (ar.succeeded()) {
          rowStreamHandler.handle(io.vertx.core.Future.succeededFuture(io.vertx.rxjava.cassandra.CassandraRowStream.newInstance(ar.result())));
        } else {
          rowStreamHandler.handle(io.vertx.core.Future.failedFuture(ar.cause()));
        }
      }
    });
    return this;
  }

    /**
   * Executes the given SQL <code>SELECT</code> statement which returns the results of the query as a read stream.
   * @param sql the SQL to execute. For example <code>SELECT * FROM table ...</code>.
   * @return current Cassandra client instance
   */
  public Single<io.vertx.rxjava.cassandra.CassandraRowStream> rxQueryStream(String sql) { 
    return Single.create(new io.vertx.rx.java.SingleOnSubscribeAdapter<>(fut -> {
      queryStream(sql, fut);
    }));
  }

  /**
   * Disconnects from the Cassandra service.
   * @return current Cassandra client instance
   */
  public io.vertx.rxjava.cassandra.CassandraClient disconnect() { 
    delegate.disconnect();
    return this;
  }

  /**
   * Disconnects from the Cassandra service.
   * @param disconnectHandler handler called when asynchronous disconnect call ends
   * @return current Cassandra client instance
   */
  public io.vertx.rxjava.cassandra.CassandraClient disconnect(Handler<AsyncResult<Void>> disconnectHandler) { 
    delegate.disconnect(disconnectHandler);
    return this;
  }

    /**
   * Disconnects from the Cassandra service.
   * @return current Cassandra client instance
   */
  public Single<Void> rxDisconnect() { 
    return Single.create(new io.vertx.rx.java.SingleOnSubscribeAdapter<>(fut -> {
      disconnect(fut);
    }));
  }

  /**
   * Execute the query and provide a handler for consuming results.
   * @param query the query to execute
   * @param resultHandler handler called when result of execution is fully fetched.
   * @return current Cassandra client instance
   */
  public io.vertx.rxjava.cassandra.CassandraClient executeWithFullFetch(String query, Handler<AsyncResult<List<Row>>> resultHandler) { 
    delegate.executeWithFullFetch(query, resultHandler);
    return this;
  }

    /**
   * Execute the query and provide a handler for consuming results.
   * @param query the query to execute
   * @return current Cassandra client instance
   */
  public Single<List<Row>> rxExecuteWithFullFetch(String query) { 
    return Single.create(new io.vertx.rx.java.SingleOnSubscribeAdapter<>(fut -> {
      executeWithFullFetch(query, fut);
    }));
  }

  /**
   * Execute the query and provide a handler for consuming results.
   * @param statement the statement to execute
   * @param resultHandler handler called when result of execution is fully fetched.
   * @return current Cassandra client instance
   */
  public io.vertx.rxjava.cassandra.CassandraClient executeWithFullFetch(Statement statement, Handler<AsyncResult<List<Row>>> resultHandler) { 
    delegate.executeWithFullFetch(statement, resultHandler);
    return this;
  }

    /**
   * Execute the query and provide a handler for consuming results.
   * @param statement the statement to execute
   * @return current Cassandra client instance
   */
  public Single<List<Row>> rxExecuteWithFullFetch(Statement statement) { 
    return Single.create(new io.vertx.rx.java.SingleOnSubscribeAdapter<>(fut -> {
      executeWithFullFetch(statement, fut);
    }));
  }

  /**
   * Execute the statement and provide a handler for consuming results.
   * @param statement the statement to execute
   * @param resultHandler handler called when result of execution is present
   * @return current Cassandra client instance
   */
  public io.vertx.rxjava.cassandra.CassandraClient execute(Statement statement, Handler<AsyncResult<io.vertx.rxjava.cassandra.ResultSet>> resultHandler) { 
    delegate.execute(statement, new Handler<AsyncResult<io.vertx.cassandra.ResultSet>>() {
      public void handle(AsyncResult<io.vertx.cassandra.ResultSet> ar) {
        if (ar.succeeded()) {
          resultHandler.handle(io.vertx.core.Future.succeededFuture(io.vertx.rxjava.cassandra.ResultSet.newInstance(ar.result())));
        } else {
          resultHandler.handle(io.vertx.core.Future.failedFuture(ar.cause()));
        }
      }
    });
    return this;
  }

    /**
   * Execute the statement and provide a handler for consuming results.
   * @param statement the statement to execute
   * @return current Cassandra client instance
   */
  public Single<io.vertx.rxjava.cassandra.ResultSet> rxExecute(Statement statement) { 
    return Single.create(new io.vertx.rx.java.SingleOnSubscribeAdapter<>(fut -> {
      execute(statement, fut);
    }));
  }

  /**
   * Prepares the provided query string.
   * @param query the query to prepare
   * @param resultHandler handler called when result of query preparation is present
   * @return current Cassandra client instance
   */
  public io.vertx.rxjava.cassandra.CassandraClient prepare(String query, Handler<AsyncResult<PreparedStatement>> resultHandler) { 
    delegate.prepare(query, resultHandler);
    return this;
  }

    /**
   * Prepares the provided query string.
   * @param query the query to prepare
   * @return current Cassandra client instance
   */
  public Single<PreparedStatement> rxPrepare(String query) { 
    return Single.create(new io.vertx.rx.java.SingleOnSubscribeAdapter<>(fut -> {
      prepare(query, fut);
    }));
  }

  /**
   * Executes the given SQL statement which returns the results of the query as a read stream.
   * @param statement the statement to execute.
   * @param rowStreamHandler the handler which is called once the operation completes. It will return an instance of {@link io.vertx.rxjava.cassandra.CassandraRowStream}.
   * @return current Cassandra client instance
   */
  public io.vertx.rxjava.cassandra.CassandraClient queryStream(Statement statement, Handler<AsyncResult<io.vertx.rxjava.cassandra.CassandraRowStream>> rowStreamHandler) { 
    delegate.queryStream(statement, new Handler<AsyncResult<io.vertx.cassandra.CassandraRowStream>>() {
      public void handle(AsyncResult<io.vertx.cassandra.CassandraRowStream> ar) {
        if (ar.succeeded()) {
          rowStreamHandler.handle(io.vertx.core.Future.succeededFuture(io.vertx.rxjava.cassandra.CassandraRowStream.newInstance(ar.result())));
        } else {
          rowStreamHandler.handle(io.vertx.core.Future.failedFuture(ar.cause()));
        }
      }
    });
    return this;
  }

    /**
   * Executes the given SQL statement which returns the results of the query as a read stream.
   * @param statement the statement to execute.
   * @return current Cassandra client instance
   */
  public Single<io.vertx.rxjava.cassandra.CassandraRowStream> rxQueryStream(Statement statement) { 
    return Single.create(new io.vertx.rx.java.SingleOnSubscribeAdapter<>(fut -> {
      queryStream(statement, fut);
    }));
  }

  /**
   * The name of the default pool
   */
  public static final String DEFAULT_POOL_NAME = io.vertx.cassandra.CassandraClient.DEFAULT_POOL_NAME;

  public static  CassandraClient newInstance(io.vertx.cassandra.CassandraClient arg) {
    return arg != null ? new CassandraClient(arg) : null;
  }
}
