/*
 * Copyright 2014 Red Hat, Inc.
 *
 * Red Hat licenses this file to you under the Apache License, version 2.0
 * (the "License"); you may not use this file except in compliance with the
 * License.  You may obtain a copy of the License at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

package io.vertx.rxjava.rabbitmq;

import java.util.Map;
import rx.Observable;
import rx.Single;
import io.vertx.core.AsyncResult;
import io.vertx.core.Handler;

/**
 * A stream of messages from a rabbitmq queue.
 *
 * <p/>
 * NOTE: This class has been automatically generated from the {@link io.vertx.rabbitmq.RabbitMQConsumer original} non RX-ified interface using Vert.x codegen.
 */

@io.vertx.lang.rx.RxGen(io.vertx.rabbitmq.RabbitMQConsumer.class)
public class RabbitMQConsumer implements io.vertx.rxjava.core.streams.ReadStream<io.vertx.rxjava.rabbitmq.RabbitMQMessage> {

  @Override
  public String toString() {
    return delegate.toString();
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) return true;
    if (o == null || getClass() != o.getClass()) return false;
    RabbitMQConsumer that = (RabbitMQConsumer) o;
    return delegate.equals(that.delegate);
  }
  
  @Override
  public int hashCode() {
    return delegate.hashCode();
  }

  public static final io.vertx.lang.rx.TypeArg<RabbitMQConsumer> __TYPE_ARG = new io.vertx.lang.rx.TypeArg<>(    obj -> new RabbitMQConsumer((io.vertx.rabbitmq.RabbitMQConsumer) obj),
    RabbitMQConsumer::getDelegate
  );

  private final io.vertx.rabbitmq.RabbitMQConsumer delegate;
  
  public RabbitMQConsumer(io.vertx.rabbitmq.RabbitMQConsumer delegate) {
    this.delegate = delegate;
  }

  public io.vertx.rabbitmq.RabbitMQConsumer getDelegate() {
    return delegate;
  }

  private rx.Observable<io.vertx.rxjava.rabbitmq.RabbitMQMessage> observable;

  public synchronized rx.Observable<io.vertx.rxjava.rabbitmq.RabbitMQMessage> toObservable() {
    if (observable == null) {
      java.util.function.Function<io.vertx.rabbitmq.RabbitMQMessage, io.vertx.rxjava.rabbitmq.RabbitMQMessage> conv = io.vertx.rxjava.rabbitmq.RabbitMQMessage::newInstance;
      observable = io.vertx.rx.java.RxHelper.toObservable(delegate, conv);
    }
    return observable;
  }

  /**
   * Fetch the specified <code>amount</code> of elements. If the <code>ReadStream</code> has been paused, reading will
   * recommence with the specified <code>amount</code> of items, otherwise the specified <code>amount</code> will
   * be added to the current stream demand.
   * @param amount 
   * @return a reference to this, so the API can be used fluently
   */
  public io.vertx.rxjava.core.streams.ReadStream<io.vertx.rxjava.rabbitmq.RabbitMQMessage> fetch(long amount) { 
    delegate.fetch(amount);
    return this;
  }

  /**
   * Set an exception handler on the read stream.
   * @param exceptionHandler the exception handler
   * @return a reference to this, so the API can be used fluently
   */
  public io.vertx.rxjava.rabbitmq.RabbitMQConsumer exceptionHandler(Handler<Throwable> exceptionHandler) { 
    delegate.exceptionHandler(exceptionHandler);
    return this;
  }

  /**
   * Set a message handler. As message appear in a queue, the handler will be called with the message.
   * @param messageArrived 
   * @return a reference to this, so the API can be used fluently
   */
  public io.vertx.rxjava.rabbitmq.RabbitMQConsumer handler(Handler<io.vertx.rxjava.rabbitmq.RabbitMQMessage> messageArrived) { 
    delegate.handler(new Handler<io.vertx.rabbitmq.RabbitMQMessage>() {
      public void handle(io.vertx.rabbitmq.RabbitMQMessage event) {
        messageArrived.handle(io.vertx.rxjava.rabbitmq.RabbitMQMessage.newInstance(event));
      }
    });
    return this;
  }

  /**
   * Pause the stream of incoming messages from queue.
   * <p>
   * The messages will continue to arrive, but they will be stored in a internal queue.
   * If the queue size would exceed the limit provided by , then incoming messages will be discarded.
   * @return a reference to this, so the API can be used fluently
   */
  public io.vertx.rxjava.rabbitmq.RabbitMQConsumer pause() { 
    delegate.pause();
    return this;
  }

  /**
   * Resume reading from a queue. Flushes internal queue.
   * @return a reference to this, so the API can be used fluently
   */
  public io.vertx.rxjava.rabbitmq.RabbitMQConsumer resume() { 
    delegate.resume();
    return this;
  }

  /**
   * Set an end handler. Once the stream has canceled successfully, the handler will be called.
   * @param endHandler 
   * @return a reference to this, so the API can be used fluently
   */
  public io.vertx.rxjava.rabbitmq.RabbitMQConsumer endHandler(Handler<Void> endHandler) { 
    delegate.endHandler(endHandler);
    return this;
  }

  /**
   * @return a consumer tag
   */
  public String consumerTag() { 
    String ret = delegate.consumerTag();
    return ret;
  }

  /**
   * Stop message consumption from a queue.
   * <p>
   * The operation is asynchronous. When consumption will be stopped, you can by notified via {@link io.vertx.rxjava.rabbitmq.RabbitMQConsumer#endHandler}
   */
  public void cancel() { 
    delegate.cancel();
  }

  /**
   * Stop message consumption from a queue.
   * <p>
   * The operation is asynchronous. When consumption will be stopped, you can by notified via {@link io.vertx.rxjava.rabbitmq.RabbitMQConsumer#endHandler}
   * @param cancelResult contains information about operation status: success/fail.
   */
  public void cancel(Handler<AsyncResult<Void>> cancelResult) { 
    delegate.cancel(cancelResult);
  }

    /**
   * Stop message consumption from a queue.
   * <p>
   * The operation is asynchronous. When consumption will be stopped, you can by notified via {@link io.vertx.rxjava.rabbitmq.RabbitMQConsumer#endHandler}
   * @return 
   * @deprecated use {@link #rxCancel} instead
   */
  @Deprecated()
  public Observable<Void> cancelObservable() { 
    io.vertx.rx.java.ObservableFuture<Void> cancelResult = io.vertx.rx.java.RxHelper.observableFuture();
    cancel(cancelResult.toHandler());
    return cancelResult;
  }

  /**
   * Stop message consumption from a queue.
   * <p>
   * The operation is asynchronous. When consumption will be stopped, you can by notified via {@link io.vertx.rxjava.rabbitmq.RabbitMQConsumer#endHandler}
   * @return 
   */
  public Single<Void> rxCancel() { 
    return Single.create(new io.vertx.rx.java.SingleOnSubscribeAdapter<>(fut -> {
      cancel(fut);
    }));
  }

  /**
   * @return is the stream paused?
   */
  public boolean isPaused() { 
    boolean ret = delegate.isPaused();
    return ret;
  }


  public static  RabbitMQConsumer newInstance(io.vertx.rabbitmq.RabbitMQConsumer arg) {
    return arg != null ? new RabbitMQConsumer(arg) : null;
  }
}
